local RUF = LibStub("AceAddon-3.0"):GetAddon("RUF")
local L = LibStub("AceLocale-3.0"):GetLocale("RUF")
local _, ns = ...
local oUF = ns.oUF
local tags = oUF.Tags.Methods or oUF.Tags
local events = oUF.TagEvents or oUF.Tags.Events
local members = {}
RUFMembers = members


-- Uppercase Name
-- Limits NPC names to 18 characters.
tags['Raeli:UpperName'] = function(unit)
	if not UnitName(unit) then return end

	-- Substitute Nickname
	local name
	if RUF.db.profile.Appearance.Text.Name.Case== 1 then 
		name = string.upper(UnitName(unit))
	elseif RUF.db.profile.Appearance.Text.Name.Case == 2 then
		name = string.lower(UnitName(unit))
	else
		name = UnitName(unit)
	end
	local player = string.upper(UnitName("player"))
	if RUF.db.char.NickName and string.upper(name) == player and string.len(RUF.db.char.NickName) > 0 then
		if RUF.db.profile.Appearance.Text.Name.Case == 1 then
			name = string.upper(RUF.db.char.NickName)
		elseif RUF.db.profile.Appearance.Text.Name.Case == 2 then
			name = string.lower(RUF.db.char.NickName)
		else
			name = RUF.db.char.NickName
		end
	end
	local r,g,b = RUF:ReturnTextColors(unit, "Name", cur, max)
	local cur, max = UnitHealth(unit), UnitHealthMax(unit)
	if RUF.db.global.TestMode == true then
		cur = math.random(75)
		max = math.random(75,100)
		val = math.random(RUF.db.char.GuildNum)
		name = RUF.db.char.Guild[val].Name
		r,g,b = RUF:ReturnTextColors(unit, "Name", cur, max,val)
	end
	
	-- Return Text String
	if UnitIsPlayer(unit) then
		return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,name)
	else
		return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,string.sub(name,1,18))
	end
end
events['Raeli:UpperName'] = 'UNIT_NAME_UPDATE UNIT_HEALTH'

-- Health
-- Class Coloured if Player, Reaction Coloured if NPC, Gradient Coloured if below 100%
tags['Raeli:FullHealth'] = function(unit)
	if not UnitName(unit) then return end
	local cur, max = UnitHealth(unit), UnitHealthMax(unit)
	if RUF.db.global.TestMode == true then
		cur = math.random(75)
		max = math.random(75,100)
	end
	local r,g,b = RUF:ReturnTextColors(unit, "Health", cur, max)
	
	-- Return Text String
	if UnitIsDead(unit) then
		if RUF.db.profile.Appearance.Text.Health.Case == 1 then
			return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,string.upper(L["Dead"]))
		elseif RUF.db.profile.Appearance.Text.Health.Case == 2 then
			return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,string.lower(L["Dead"]))
		else
			return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,L["Dead"])
		end		
	elseif UnitIsGhost(unit) then
		if RUF.db.profile.Appearance.Text.Health.Case == 1 then
			return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,string.upper(L["Ghost"]))
		elseif RUF.db.profile.Appearance.Text.Health.Case == 2 then
			return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,string.lower(L["Ghost"]))
		else
			return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,L["Ghost"])
		end
	elseif not UnitIsConnected(unit) then
		if RUF.db.profile.Appearance.Text.Health.Case == 1 then
			return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,string.upper(L["Offline"]))
		elseif RUF.db.profile.Appearance.Text.Health.Case == 2 then
			return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,string.lower(L["Offline"]))
		else
			return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,L["Offline"])
		end
	elseif cur == max then -- if we're at full health 
		if RUF.db.profile.Appearance.Text.Health.Case == 1 then
			return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,RUF:Short(cur,true))
		elseif RUF.db.profile.Appearance.Text.Health.Case == 2 then
			return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,string.lower(RUF:Short(cur,true)))
		else
			return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,RUF:Short(cur,true))
		end		
	else
		if RUF.db.profile.Appearance.Text.Health.Case == 1 then
			return string.format('|cff%02x%02x%02x%s%s%s%%|r',r*255,g*255,b*255,RUF:Short(cur,true), " - ",RUF:Percent(cur,max))
		elseif RUF.db.profile.Appearance.Text.Health.Case == 2 then
			return string.format('|cff%02x%02x%02x%s%s%s%%|r',r*255,g*255,b*255,(RUF:Short(cur,true)), " - ",RUF:Percent(cur,max))
		else
			return string.format('|cff%02x%02x%02x%s%s%s%%|r',r*255,g*255,b*255,string.lower(RUF:Short(cur,true)), " - ",RUF:Percent(cur,max))
		end		
	end
end
events['Raeli:FullHealth'] = 'UNIT_HEALTH UNIT_CONNECTION'

 
tags['Raeli:Mana'] = function(unit) -- Mana for Shadow Priests
	--local cur, max = UnitPower(unit,0), UnitPowerMax(unit,0)
	local pType,pToken,altr,altg,altb = UnitPowerType(unit)
	--if (altr ~= nil) or (altg ~= nil) or (altb ~= nil) then
	--	local pr,pg,pb = altr or 0,altg or 0,altb or 0
	--else
		local cur, max = UnitPower(unit,0), UnitPowerMax(unit,0)
		local r,g,b = RUF:ReturnTextColors(unit, "Class", cur, max)
		--local pr,pg,pb = unpack(RUF.db.profile.Colors.PowerColors[pType])
	--end
	if RUF.db.global.TestMode == true then
		cur = math.random(75)
		max = math.random(75,100)
	end

	-- ERROR WITH VEHICLES. TRY IF UNIT == VEHICLE or something like that.
	if unit == "vehicle" then unit = "player" end
	if RUF.db.profile.unit[unit].Frame.Text.Class.Enabled == 0 then -- Always Hidden on this unit.
		return ''
	elseif RUF.db.profile.unit[unit].Frame.Text.Class.Enabled == 1 then -- Hidden at 0
		if max < 1 then
			return ''
		elseif cur > 0 and cur < max then
			if RUF.db.profile.Appearance.Text.Class.Case == 2 then
				return string.format('|cff%02x%02x%02x%s%s%s%%|r',r*255,g*255,b*255,string.lower(RUF:Short(cur,true)), " - ",RUF:Percent(cur,max))
			else
				return string.format('|cff%02x%02x%02x%s%s%s%%|r',r*255,g*255,b*255,RUF:Short(cur,true), " - ",RUF:Percent(cur,max))
			end	
		elseif cur == max then
			if RUF.db.profile.Appearance.Text.Class.Case == 2 then
				return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,string.lower(RUF:Short(cur,true)))
			else
				return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,RUF:Short(cur,true))
			end
		else
			return ''			
		end
	elseif 	RUF.db.profile.unit[unit].Frame.Text.Class.Enabled == 2 then -- Always Show
		if cur == max then
			if RUF.db.profile.Appearance.Text.Class.Case == 2 then
				return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,string.lower(RUF:Short(cur,true)))
			else
				return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,RUF:Short(cur,true))
			end
		elseif cur < max and cur > 0 then
			if RUF.db.profile.Appearance.Text.Class.Case == 2 then
				return string.format('|cff%02x%02x%02x%s%s%s%%|r',r*255,g*255,b*255,string.lower(RUF:Short(cur,true)), " - ",RUF:Percent(cur,max))
			else
				return string.format('|cff%02x%02x%02x%s%s%s%%|r',r*255,g*255,b*255,RUF:Short(cur,true), " - ",RUF:Percent(cur,max))
			end
		else
			if RUF.db.profile.Appearance.Text.Class.Case == 2 then
				return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,"0")
			else
				return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,"0")
			end
		end
	end

	
	--[[if max == 0 then
		return ''
	elseif cur == max then
		return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,RUF:Short(cur,true))
	elseif cur < max and cur > 0 then
		return string.format('|cff%02x%02x%02x%s%s%s%%|r',r*255,g*255,b*255,RUF:Short(cur,true), " - ",RUF:Percent(cur,max))
	else
		return ''
	end]]--

end
events['Raeli:Mana'] = 'UNIT_POWER'


-- Power value
-- Mana, Rage, Insanity, Maelstrom etc.
-- Percentage shown if below 100%, hidden if 0
tags['Raeli:Power'] = function(unit,realunit)
	local cur, max = UnitPower(unit), UnitPowerMax(unit)
	local pType,pToken,altr,altg,altb = UnitPowerType(unit)
	--if (altr ~= nil) or (altg ~= nil) or (altb ~= nil) then
	--	local pr,pg,pb = altr or 0,altg or 0,altb or 0
	--else
		local cur, max = UnitPower(unit), UnitPowerMax(unit)
		local r,g,b = RUF:ReturnTextColors(unit, "Power", cur, max)
		--local pr,pg,pb = unpack(RUF.db.profile.Colors.PowerColors[pType])
	--end
	if RUF.db.global.TestMode == true then
		cur = math.random(75)
		max = math.random(75,100)
	end


	-- ERROR WITH VEHICLES. TRY IF UNIT == VEHICLE or something like that.
	if unit == "vehicle" then unit = "player" end
	if RUF.db.profile.unit[unit].Frame.Text.Power.Enabled == 0 then -- Always Hidden on this unit.
		return ''
	elseif RUF.db.profile.unit[unit].Frame.Text.Power.Enabled == 1 then -- Hidden at 0
		if max < 1 then
			return ''
		elseif cur > 0 and cur < max then
			if RUF.db.profile.Appearance.Text.Power.Case == 2 then
				return string.format('|cff%02x%02x%02x%s%s%s%%|r',r*255,g*255,b*255,string.lower(RUF:Short(cur,true)), " - ",RUF:Percent(cur,max))
			else
				return string.format('|cff%02x%02x%02x%s%s%s%%|r',r*255,g*255,b*255,RUF:Short(cur,true), " - ",RUF:Percent(cur,max))
			end	
		elseif cur == max then
			if RUF.db.profile.Appearance.Text.Power.Case == 2 then
				return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,string.lower(RUF:Short(cur,true)))
			else
				return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,RUF:Short(cur,true))
			end
		else
			return ''			
		end
	elseif 	RUF.db.profile.unit[unit].Frame.Text.Power.Enabled == 2 then -- Always Show
		if cur == max then
			if RUF.db.profile.Appearance.Text.Power.Case == 2 then
				return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,string.lower(RUF:Short(cur,true)))
			else
				return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,RUF:Short(cur,true))
			end
		elseif cur < max and cur > 0 then
			if RUF.db.profile.Appearance.Text.Power.Case == 2 then
				return string.format('|cff%02x%02x%02x%s%s%s%%|r',r*255,g*255,b*255,string.lower(RUF:Short(cur,true)), " - ",RUF:Percent(cur,max))
			else
				return string.format('|cff%02x%02x%02x%s%s%s%%|r',r*255,g*255,b*255,RUF:Short(cur,true), " - ",RUF:Percent(cur,max))
			end
		else
			if RUF.db.profile.Appearance.Text.Power.Case == 2 then
				return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,"0")
			else
				return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,"0")
			end
		end
	end

	
	--[[if max == 0 then
		return ''
	elseif cur == max then
		return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,RUF:Short(cur,true))
	elseif cur < max and cur > 0 then
		return string.format('|cff%02x%02x%02x%s%s%s%%|r',r*255,g*255,b*255,RUF:Short(cur,true), " - ",RUF:Percent(cur,max))
	else
		return ''
	end]]--

end
events['Raeli:Power'] = 'UNIT_POWER'


-- Unit smart level
tags['Raeli:Level'] = function(unit)
	local l = UnitLevel(unit)
	if RUF.db.global.TestMode == true then
		l = math.random(120)
	end
	local r,g,b = RUF:GetLevelColor(l)
	if l <= 0 then l = "??" end  
	if l ~= UnitLevel("player") then
		return string.format('|cff%02x%02x%02x%s|r',r*255,g*255,b*255,l)
	else
		return ''
	end
end
events['Raeli:Level'] = 'UNIT_LEVEL PLAYER_LEVEL_UP UNIT_CLASSIFICATION_CHANGED'
oUF.Tags.SharedEvents.EVENT_NAME = true


tags['Raeli:AFKDND'] = function(unit)
	if UnitIsAFK(unit) then
		return string.format('|cffFFFFFF[AFK]|r')
	end
	if UnitIsDND(unit) then
		return string.format('|cffFFFFFF[DND]|r')
	end
end
events['Raeli:AFKDND'] = 'PLAYER_FLAGS_CHANGED'




--[[
if RUF.db.profile.unit[unit].Bars.Power == 0 and element.__owner.Power:IsShown() then -- Always Hidden on this unit.
	element.__owner.Power:Hide()
	element.__owner.Power.bg:Hide()
elseif RUF.db.profile.unit[unit].Bars.Power == 1 then -- Hidden at 0
	if cur < 1 and element.__owner.Power:IsShown() then
		element.__owner.Power:Hide()
		element.__owner.Power.bg:Hide()
	end
	if cur > 0 and not element.__owner.Power:IsShown() then 
		element.__owner.Power:Show()
		element.__owner.Power.bg:Show() 
	end
elseif 	RUF.db.profile.unit[unit].Bars.Power == 2 and not element.__owner.Power:IsShown() then		
	element.__owner.Power:Show()
	element.__owner.Power.bg:Show() 	
end]]--
